<?php

namespace The99Bitcoins\EthMiningCalculator;

class Plugin {
	protected $config
		= array(
			'url'       => '99bitcoins-eth-mining-calculator',
			'plugin'    => '99bitcoins-eth-mining-calculator/',
			'templates' => '../templates/',
		);

	public function __construct( $config = array() ) {
		$this->config = $config + $this->config;
		add_action( 'admin_menu', array( $this, 'backendActionMenu' ) );
		add_action( 'init', array( $this, 'init' ) );
	}

	public function init() {
		add_shortcode( 'eth-mining-calculator', [ $this, 'frontendShortcodeMiningCalculator' ] );
		add_shortcode( 'eth-price', [ $this, 'frontendShortcodePrice' ] );

		$locale = get_locale();
		load_textdomain( '99btc-emc', WP_LANG_DIR . '/plugins/99btc-emc-' . $locale . '.mo' );
		load_plugin_textdomain( '99btc-emc', false, $this->config['plugin'] . 'languages/' );
	}

	public static function cron() {
		$curl = new \WP_Http_Curl();
		$data = $curl->request( 'https://99bitcoins.com/eth.json' );
		if ( $data['response']['code'] == 200 ) {
			$data = json_decode( $data['body'], true );

			$options                      = get_option( __CLASS__ );
			$options['eth_difficulty']    = $data['difficulty'];
			$options['eth_block_time']    = $data['block_time'];
			$options['eth_exchange_rate'] = $data['price_usd'];
			update_option( __CLASS__, $options, 'no' );
		}
	}

	public function backendActionMenu() {
		add_menu_page( translate( 'Ethereum mining calculator', '99btc-emc' ),
			translate( 'Ethereum mining calculator', '99btc-emc' ), 'manage_options', $this->config['url'],
			array( $this, 'backendPageConfig' ), '', null );
		add_submenu_page( $this->config['url'], translate( 'Config', '99btc-emc' ), translate( 'Config', '99btc-emc' ),
			'manage_options', $this->config['url'], array( $this, 'backendPageConfig' ) );
		add_submenu_page( $this->config['url'], translate( 'Tooltips', '99btc-emc' ),
			translate( 'Tooltips', '99btc-emc' ), 'manage_options', $this->config['url'] . '/tooltips',
			array( $this, 'backendPageTooltips' ) );
		add_submenu_page( $this->config['url'], translate( 'How to use', '99btc-emc' ),
			translate( 'How to use', '99btc-emc' ), 'manage_options', $this->config['url'] . '/how-to',
			array( $this, 'backendPageHowToUse' ) );
	}

	public function backendPageConfig() {
		$variables = get_option( __CLASS__ );
		if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			$options = get_option( __CLASS__ );
			if ( isset( $_POST['eth_difficulty'] ) ) {
				$options['eth_difficulty'] = $_POST['eth_difficulty'];
			}
			if ( isset( $_POST['eth_block_time'] ) ) {
				$options['eth_block_time'] = $_POST['eth_block_time'];
			}
			if ( isset( $_POST['eth_exchange_rate'] ) ) {
				$options['eth_exchange_rate'] = $_POST['eth_exchange_rate'];
			}
			if ( isset( $_POST['eth_hide_copyright'] ) ) {
				$options['eth_hide_copyright'] = $_POST['eth_hide_copyright'];
			}
			update_option( __CLASS__, $options, 'no' );
			$variables                     = array_merge( $variables, $options );
			$variables['notice_css_class'] = 'updated notice';
			$variables['notice_message']   = 'Changes have been saved';
		}

		$this->render( 'config', $variables );
	}

	public function backendPageTooltips() {
		$variables           = get_option( __CLASS__ );
		$variables['fields'] = array(
			array(
				'name'        => 'eth_tooltip_hash_rate',
				'title'       => 'Hash rate',
				'placeholder' => 'Tooltip for hash rate',
			),
			array(
				'name'        => 'eth_tooltip_difficulty',
				'title'       => 'Difficulty factor',
				'placeholder' => 'Tooltip for difficulty factor',
			),
			array(
				'name'        => 'eth_tooltip_block_time',
				'title'       => 'Block time in seconds',
				'placeholder' => 'Tooltip for block time in seconds',
			),
			array(
				'name'        => 'eth_tooltip_exchange_rate',
				'title'       => 'USD/ETH exchange rate',
				'placeholder' => 'Tooltip for USD/ETH exchange rate',
			),
			array(
				'name'        => 'eth_tooltip_power_watts',
				'title'       => 'Power (Watts)',
				'placeholder' => 'Tooltip for power',
			),
			array(
				'name'        => 'eth_tooltip_power_cost',
				'title'       => 'Power Cost (USD/kWh)',
				'placeholder' => 'Tooltip for power cost',
			),
			array(
				'name'        => 'eth_tooltip_hardware_costs',
				'title'       => 'Hardware Costs (USD)',
				'placeholder' => 'Tooltip for hardware costs',
			),
		);

		if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			$options = get_option( __CLASS__ );
			foreach ( $variables['fields'] as $field ) {
				if ( isset( $_POST[ $field['name'] ] ) ) {
					$options[ $field['name'] ] = stripslashes( $_POST[ $field['name'] ] );
				}
			}
			update_option( __CLASS__, $options, 'no' );
			$variables                     = array_merge( $variables, $options );
			$variables['notice_css_class'] = 'updated notice';
			$variables['notice_message']   = 'Changes have been saved';
		}
		$this->render( 'tooltips', $variables );
	}

	public function backendPageHowToUse() {
		$this->render( 'how-to-use' );
	}

	public function frontendShortcodeMiningCalculator() {
		$variables    = get_option( __CLASS__ );
		$this->page   = 'miningCalculator';
		$this->layout = 'empty';

		$variables['hash_grades']                 = [
			'KH/s' => '1000',
			'MH/s' => '1000000',
			'GH/s' => '1000000000',
			'TH/s' => '1000000000000',
			'PH/s' => '1000000000000000',
		];
		$variables['days']                        = [
			'1 day'   => '1',
			'1 week'  => '7',
			'2 weeks' => '14',
			'3 weeks' => '21',
			'1 month' => '30',
		];
		$variables['eth_hash_rate_grade']         = 'MH/s';
		$variables['eth_network_hash_rate_grade'] = 'GH/s';
		$variables['result']                      = [ ];

		$variables['eth_network_hash_rate'] = bcdiv( $variables['eth_network_hash_rate'],
			$variables['hash_grades'][ $variables['eth_network_hash_rate_grade'] ], 5 );

		if ( isset( $_GET['eth_action'] ) && $_GET['eth_action'] == 'calculate' ) {
			if ( ! empty( $_GET['eth_hash_rate'] ) ) {
				$variables['eth_hash_rate'] = $_GET['eth_hash_rate'];
			}
			if ( ! empty( $_GET['eth_hash_rate_grade'] ) ) {
				$variables['eth_hash_rate_grade'] = $_GET['eth_hash_rate_grade'];
			}
			if ( ! empty( $_GET['eth_network_hash_rate'] ) ) {
				$variables['eth_network_hash_rate'] = $_GET['eth_network_hash_rate'];
			}
			if ( ! empty( $_GET['eth_network_hash_rate_grade'] ) ) {
				$variables['eth_network_hash_rate_grade'] = $_GET['eth_network_hash_rate_grade'];
			}
			if ( ! empty( $_GET['eth_difficulty'] ) ) {
				$variables['eth_difficulty'] = $_GET['eth_difficulty'];
			}
			if ( ! empty( $_GET['eth_block_time'] ) ) {
				$variables['eth_block_time'] = $_GET['eth_block_time'];
			}
			if ( ! empty( $_GET['eth_exchange_rate'] ) ) {
				$variables['eth_exchange_rate'] = $_GET['eth_exchange_rate'];
			}
			if ( ! empty( $_GET['eth_power_watts'] ) ) {
				$variables['eth_power_watts'] = $_GET['eth_power_watts'];
			}
			if ( ! empty( $_GET['eth_power_cost'] ) ) {
				$variables['eth_power_cost'] = $_GET['eth_power_cost'];
			}
			if ( ! empty( $_GET['eth_hardware_costs'] ) && is_numeric( $_GET['eth_hardware_costs'] ) ) {
				$variables['eth_hardware_costs'] = $_GET['eth_hardware_costs'];
			}
			if ( is_numeric( $variables['eth_difficulty'] ) ) {
				$networkHash  = bcdiv( $variables['eth_difficulty'], $variables['eth_block_time'], 10 );
				$hashRate     = bcmul( $variables['eth_hash_rate'],
					$variables['hash_grades'][ $variables['eth_hash_rate_grade'] ], 10 );
				$userRatio    = bcdiv( $hashRate, $networkHash, 10 );
				$blocksPerMin = bcdiv( 60, $variables['eth_block_time'], 10 );
				$ethPerDay    = bcmul( $blocksPerMin, 5, 10 );
				$ethPerDay    = bcmul( $ethPerDay, $userRatio, 10 );
				$ethPerDay    = bcmul( $ethPerDay, 60, 10 );
				$ethPerDay    = bcmul( $ethPerDay, 24, 10 );
				$btcOptions   = get_option( 'The99Bitcoins\BtcMiningCalculator\Plugin' );
				$usdBtc       = 0;
				if ( $btcOptions && ! empty( $btcOptions['btc_exchange_rate'] ) ) {
					$usdBtc = $btcOptions['btc_exchange_rate'];
				}
				$variables['usd_btc'] = $usdBtc;
				foreach ( $variables['days'] as $k => $days ) {
					$variables['result'][ $k ]['earning']         = bcmul( $ethPerDay, $days, 8 );
					$variables['result'][ $k ]['fee']             = bcmul( $variables['result'][ $k ]['earning'],
						$variables['pool_fee'] / 100, 8 );
					$variables['result'][ $k ]['profit_currency'] = bcsub( $variables['result'][ $k ]['earning'],
						$variables['result'][ $k ]['fee'], 8 );
					$variables['result'][ $k ]['profit_usd']
					                                              = bcmul( $variables['result'][ $k ]['profit_currency'],
						$variables['eth_exchange_rate'], 2 );
					if ( $variables['eth_hardware_costs'] ) {
						$variables['result'][ $k ]['profit_usd'] = bcsub( $variables['result'][ $k ]['profit_usd'],
							$variables['eth_hardware_costs'], 2 );
					}
					if ( $variables['eth_power_watts'] && $variables['eth_power_cost'] ) {
						$variables['result'][ $k ]['power_kwh']   = bcdiv( $days, 1000, 10 );
						$variables['result'][ $k ]['power_kwh']   = bcmul( '24',
							$variables['result'][ $k ]['power_kwh'], 10 );
						$variables['result'][ $k ]['power_kwh']   = bcmul( $variables['eth_power_watts'],
							$variables['result'][ $k ]['power_kwh'], 10 );
						$variables['result'][ $k ]['power_price'] = bcmul( $variables['result'][ $k ]['power_kwh'],
							$variables['eth_power_cost'], 2 );
						$variables['result'][ $k ]['profit_usd']  = bcsub( $variables['result'][ $k ]['profit_usd'],
							$variables['result'][ $k ]['power_price'], 2 );
					}
					$variables['result'][ $k ]['profit_btc'] = bcdiv( $variables['result'][ $k ]['profit_usd'], $usdBtc,
						8 );
				}
			}
		}

		ob_start();
		$this->render( 'mining-calculator', $variables );
		$content = ob_get_contents();
		ob_end_clean();

		return trim( $content );
	}

	public function frontendShortcodePrice( $args, $body, $code ) {
		$options = get_option( __CLASS__ );

		return number_format( $options['eth_exchange_rate'], 2, '.', ',' );
	}

	protected function render( $_template, $_vairables = array(), $_layout = '' ) {
		extract( $_vairables );
		ob_start();
		include $this->config['templates'] . $_template . '.php';
		$_content = ob_get_contents();
		ob_end_clean();
		if ( $_layout ) {
			include $this->config['tempaltes'] . 'layout' . DIRECTORY_SEPARATOR . $_layout . '.php';
		} else {
			echo $_content;
		}
	}
}
